import logging
import base64
import time


def encode_string(txt):
    """Base64 Encode String"""
    try:
        encoded_string = base64.b64encode(txt.encode())
    except Exception as e:
        logging.error(str(e))
    else:
        return encoded_string
    return False


def current_timestamp(ms=True):
    """Retrieves up to milliseconds when set to true.
    Otherwise return to second
    """
    to_second = time.time()
    if ms:
        return int((to_second + 0.5) * 1000)
    return to_second


def convert_datetime_to_int(dt_time):
    """Convert date time object to integer"""
    return int(dt_time.strftime("%Y%m%d%H%M%S"))



from unittest.mock import MagicMock

from tests.base import BaseTest


class TestStkPush(BaseTest):
    def test_stk_push(self):
        amount = 10  # You can change this to any amount in KSH
        data = {
            "BusinessShortCode": self.short_code,
            "Password": self.password,
            "Timestamp": self.current_timestamp,
            "TransactionType": "CustomerPayBillOnline",
            "Amount": amount,
            "PartyA": self.mobile_number,
            "PartyB": self.short_code,
            "PhoneNumber": self.mobile_number,
            "CallBackURL": self.callback_url,
            "AccountReference": self.order_id,
            "TransactionDesc": "CustomerBuyGoodsOnline"
        }
        result = {
            "MerchantRequestID": "14938-14045528-1",
            "CheckoutRequestID": "ws_CO_300420201828468847",
            "ResponseCode": "0",
            "ResponseDescription": "Success. Request accepted for processing",
            "CustomerMessage": "Success. Request accepted for processing"
        }
        self.instance.lipa_na_mpesa_online_payment = MagicMock(
            return_value=(result, 200))
        body, status_code = self.instance.lipa_na_mpesa_online_payment(data)
        assert "MerchantRequestID" in body
        assert "CheckoutRequestID" in body
        assert "ResponseDescription" in body
        assert "ResponseCode" in body
        assert "CustomerMessage" in body
        assert status_code == 200

    def test_stk_confirm(self):
        checkout_request_id = "ws_CO_300420201835238048"
        data = {
            "BusinessShortCode": self.short_code,
            "Password": self.password,
            "Timestamp": self.current_timestamp,
            "CheckoutRequestID": checkout_request_id
        }
        result = {
            "ResponseCode": "0",
            "ResponseDescription": "The service request has been accepted successfully",
            "MerchantRequestID": "15777-13928452-1",
            "CheckoutRequestID": "ws_CO_300420201835238048",
            "ResultCode": "0",
            "ResultDesc": "The service request is processed successfully."
        }
        self.instance.lipa_na_mpesa_online_query = MagicMock(
            return_value=(result, 200))
        body, status_code = self.instance.lipa_na_mpesa_online_query(data)
        assert "MerchantRequestID" in body
        assert "CheckoutRequestID" in body
        assert "ResponseCode" in body
        assert "ResultDesc" in body
        assert "ResponseDescription" in body
        assert "ResultCode" in body
        assert status_code == 200



import datetime
import os

from dotenv import load_dotenv
from mpesa_sdk.gateway import oauth_generate_token, Mpesa
from mpesa_sdk.helpers import utils

load_dotenv()


class BaseTest:
    @classmethod
    def setup_class(cls):
        current_date_time = datetime.datetime.today()
        cls.passkey = os.environ.get("SAFARICOM_LIPA_NA_MPESA_PASSKEY")
        cls.consumer_key = os.environ.get("SAFARICOM_CONSUMER_KEY")
        cls.consumer_secret = os.environ.get("SAFARICOM_CONSUMER_SECRET")
        cls.short_code = int(os.environ.get("SAFARICOM_SHORT_CODE"))
        cls.mobile_number = os.environ.get("SAFARICOM_TEST_MOBILE_NUMBER")
        cls.current_timestamp = str(utils.convert_datetime_to_int(current_date_time))
        cls.order_id = "11223344"
        cls.callback_url = os.environ.get("SAFARICOM_CALL_BACK_URL")
        cls.password = utils.encode_string(
            str(cls.short_code) + cls.passkey + str(cls.current_timestamp))\
            .decode()
        result, status_code = oauth_generate_token(
            consumer_key=cls.consumer_key,
            consumer_secret=cls.consumer_secret,
            grant_type="client_credentials",
            env="sandbox")
        cls.instance = Mpesa(result["access_token"], env="sandbox")
